@@ -16,14 +16,12 @@
 
 import json
 import sys
+from typing import Union
 from urllib.request import HTTPError
 
+from railib.api import TransactionAsyncResponse
 
-__all__ = [
-    "http_error"
-    "problems"
-    "results"
-]
+__all__ = ["http_error", "problems", "results"]
 
 
 def http_error(e: HTTPError) -> None:
@@ -33,43 +31,46 @@ def http_error(e: HTTPError) -> None:
         print(json.dumps(rsp, indent=2))
 
 
-def _show_row(row: list, end='\n'):
+def _show_row(row: list, end="\n"):
     row = [f'"{item}"' if isinstance(item, str) else str(item) for item in row]
-    row = ', '.join(row)
+    row = ", ".join(row)
     print(row, end=end)
 
 
 # Print query response outputs as rel relations.
 def _show_rel(rsp: dict) -> None:
-    if rsp["aborted"]:
+    if rsp.get("aborted", False):
         print("aborted")
         return
-    outputs = rsp["output"]
-    if len(outputs) == 0:
-        print("false")
-        return
-    count = 0
-    for output in outputs:
-        cols = output["columns"]
-        rkey = output["rel_key"]
-        name = rkey["name"]
-        keys = rkey["keys"]
-        if name == "abort" and len(cols) == 0:
-            continue  # ignore constraint results
-        if count > 0:
+    if rsp.get("output", False):
+        outputs = rsp["output"]
+        if len(outputs) == 0:
+            print("false")
+            return
+        count = 0
+        for output in outputs:
+            cols = output["columns"]
+            rkey = output["rel_key"]
+            name = rkey["name"]
+            keys = rkey["keys"]
+            if name == "abort" and len(cols) == 0:
+                continue  # ignore constraint results
+            if count > 0:
+                print()
+            sig = "*".join(keys)
+            print(f"// {name} ({sig})")
+            rows = list(zip(*cols))
+            if len(rows) == 0:
+                print("true")
+                continue
+            for i, row in enumerate(rows):
+                if i > 0:
+                    print(";")
+                _show_row(row, end="")
             print()
-        sig = '*'.join(keys)
-        print(f"// {name} ({sig})")
-        rows = list(zip(*cols))
-        if len(rows) == 0:
-            print("true")
-            continue
-        for i, row in enumerate(rows):
-            if i > 0:
-                print(";")
-            _show_row(row, end='')
-        print()
-        count += 1
+            count += 1
+    if rsp.get("status", False):
+        print(rsp["status"])
 
 
 # Print the problems in the given response dict.
@@ -93,7 +94,7 @@ def problems(rsp: dict) -> None:
 
 
 # Print the results contained in the given response dict.
-def results(rsp: dict, format="physical") -> None:
+def results(rsp: Union[dict, list], format="physical") -> None:
     if rsp is None:
         return
     if format == "wire":
@@ -103,3 +104,17 @@ def results(rsp: dict, format="physical") -> None:
         problems(rsp)
     else:
         raise Exception(f"unknown format: '{format}'")
+
+
+# Print the results contained in the given TransactionAsyncResponse.
+def results(rsp: TransactionAsyncResponse) -> None:
+    if rsp.results is None:
+        return
+
+    for i, res in enumerate(rsp.results):
+        print(res["relationId"])
+        for v in zip(*res["table"].to_pydict().values()):
+            print(v)
+
+        if i < len(rsp.results) - 1:
+            print()
